﻿using System;
using System.Collections.Generic;
using System.Data;
using System.Dynamic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Unibase.Data;
using Unibase.Data.Enums;
using Unibase.Data.Expressions;
using Unibase.Dependency;
using Unibase.Platform.Data;
using Unibase.Platform.DataSources.Attributes;
using Unibase.Platform.Managers;
using Unibase.Data.Sql;

namespace Bizgaze.CRM.Visitors.Managers
{
    internal partial class VisitorManager : BaseManager, IVisitorManager
    {
        #region Getting data By VisitorId
        // This method gets a visitor record with the given VisitorId. It takes a SelectRequest as parameter and returns a SelectResponse<Infos.Visitor> object.
        public async Task<SelectResponse<Infos.Visitor>> GetVisitorByIdAsync(SelectRequest request, long VisitorId)
        {
            var exp = request.Filter();
            exp = exp & Unibase.Data.Expressions.Exp.Eq(Infos.Visitor.Property.VisitorId.ToString(), VisitorId);
            request.Filter(exp);
            var data = await GetSingleAsync<Infos.Visitor>(request);
            return data;
        }
        #endregion

        #region Get Visitors html
        //This method is used to get the html of a visitor. It takes a Visitor ID as an argument, then gets the related templates from Unibase.Platform.Templates.Managers.ITemplateManager, if the templates are found the id is assigned to the variable "templateid" and the information from GetVisitorByIdAsync is used to populate the fields in the template. In the end, the PrintTemplate object is returned with the PrintHTML.
        [SelectMethod(ApiName = "Get visitors Html", AppName = "crm", Description = "Get visitors Html", UniqueName = "bizgaze_transact_visitors_GetvisitorsHtml", MethodName = "getvisitorshtml", ModuleName = "visitors", PackageName = "bizgaze", ParamName1 = "visitorid")]
        public async Task<SelectResponse<Unibase.Platform.Templates.Infos.PrintTemplate>> GetVisitorshtmlAsync(SelectRequest srequest, long VisitorId)
        {
            long templateid = 0;
            //var appManager = Unibase.Dependency.DependencyConfig.Resolve<Unibase.Platform.Apps.Managers.IAppManager>();
            var templateManager = Unibase.Dependency.DependencyConfig.Resolve<Unibase.Platform.Templates.Managers.ITemplateManager>();
            var templateInfos = await templateManager.GetTemplatesAsync(srequest, "Bizgaze_Extension_CRM_CRM_TemplateProvider_Visitors");
            if (templateInfos.Result != null && templateInfos.Result.Count > 0)
            {
                templateid = templateInfos.Result[0].TemplateId;
            }
            var result = new SelectResponse<Unibase.Platform.Templates.Infos.PrintTemplate>();
            var printTemplate = new Unibase.Platform.Templates.Infos.PrintTemplate();
            //var templateManager = //Unibase.Dependency.DependencyConfig.Resolve<Unibase.Platform.Templates.Managers.ITemplateManager>();
            var _dbContext = DependencyConfig.Resolve<Unibase.Data.IDbContext>();
            Unibase.Platform.Templates.Helpers.TemplateHelper helper = new Unibase.Platform.Templates.Helpers.TemplateHelper(_dbContext);
                var VisitorInfo_ = await GetVisitorByIdAsync(srequest, VisitorId);
            if (VisitorInfo_.Result != null)
            {
                var tInfo = await templateManager.GetTemplateAsync(srequest, templateid);
                var templateInfo = tInfo.Result;
                if (templateInfo == null)
                {
                    return SelectResponse<Unibase.Platform.Templates.Infos.PrintTemplate>.CreateErrorResponse("1", "Template Not Found");
                }
                string CreatedDate = VisitorInfo_.Result.CheckInDate.ToString("dd-MM-yyyy HH:mm:ss");
                var template = System.Web.HttpUtility.UrlDecode(templateInfo.TemplateText.ToString());
                string multiTemplateText = "";
                if (template.Contains("{{fullname}}"))
                {
                    template = template.Replace("{{fullname}}", VisitorInfo_.Result.FullName);
                }
                if (template.Contains("{{comingfrom}}"))
                {
                    template = template.Replace("{{comingfrom}}", VisitorInfo_.Result.ComingFrom);
                }
                if (template.Contains("{{purposeofvisit}}"))
                {
                    template = template.Replace("{{purposeofvisit}}", VisitorInfo_.Result.PurposeOfVisitType);
                }
                if (template.Contains("{{phonenumber}}"))
                {
                    template = template.Replace("{{phonenumber}}", VisitorInfo_.Result.MobileNumber);
                }
                if (template.Contains("{{code}}"))
                {
                    template = template.Replace("{{code}}", VisitorInfo_.Result.Code);
                }
                if (template.Contains("{{createddate}}"))
                {
                    template = template.Replace("{{createddate}}", CreatedDate.ToString());
                }
                if (template.Contains("{{photourl}}"))
                {
                    template = template.Replace("{{photourl}}", VisitorInfo_.Result.PhotoUrl);
                }
                if (template.Contains("{{HostName}}"))
                {
                    template = template.Replace("{{HostName}}", VisitorInfo_.Result.HostName);
                }
                if (template.Contains("{{imageurl}}"))
                {
                    template = template.Replace("{{imageurl}}", VisitorInfo_.Result.ImageUrl);
                }

                multiTemplateText = template;

                printTemplate.PrintHTML = multiTemplateText;
                printTemplate.FileName = "Visitors";
                printTemplate.PkId = VisitorId;
                result.Result = printTemplate;
                return result;
            }
            else
            {
                return SelectResponse<Unibase.Platform.Templates.Infos.PrintTemplate>.CreateErrorResponse("1", "visitor  Not Found");
            }

        }
        #endregion

        #region Get Pre_Visitors
        /** 
         * This method is used to get the pre-visitors with the specified code.  
         * If the visitor with the given code is present, it returns the visitor data else it returns an error. 
         * @param srequest The select request object 
         * @param code The code of the visitor 
         * @return The select response object containing the visitor data or the error. 
         */
        [SelectMethod(ApiName = "Get  pre visitors", AppName = "crm", Description = "Get pre visitors", UniqueName = "bizgaze_transact_visitors_Getprevisitors", MethodName = "Getprevisitors", ModuleName = "visitors", PackageName = "bizgaze", ParamName1 = "code")]
        public async Task<SelectResponse<Bizgaze.CRM.Visitors.Infos.Visitor>> GetpreVisitorsAsync(SelectRequest srequest, string code)
        {
            var exp = srequest.Filter();
            exp &= Exp.Eq(Bizgaze.CRM.Visitors.Infos.Visitor.Property.Code.ToString(), code);
            srequest.Filter(exp);
            var visData = await GetSingleAsync<Bizgaze.CRM.Visitors.Infos.Visitor>(srequest);

            if (visData.Result != null)
            {
                var refStatusId = visData.Result.RefStatusId;
                if (refStatusId != 2)
                {
                    return visData;

                }
            }
            else
            {
                return SelectResponse<Bizgaze.CRM.Visitors.Infos.Visitor>.CreateErrorResponse("404", "Please Enter A Valid Uniqueid!");

            }
            return SelectResponse<Bizgaze.CRM.Visitors.Infos.Visitor>.CreateErrorResponse("404", "you have been already checked out");

        }
        #endregion

        #region Visitor Agreement
        /* 
         Action Method Name: visitoragreement 
         Description: This action method is used to update the agreement status of the visitors in the bizgazecrm_visitors table. 
         Parameters: visitorid - The ID of the visitor whose status is to be updated. 
         Return: ActionResponse - Returns success or error response based on the success of the query. 
         */
        [ActionMethod(ApiName = "visitor agreement", AppName = "crm", Description = "visitor agreement", UniqueName = "bizgaze_transact_visitors_visitoragreement", MethodName = "visitoragreement", ModuleName = "visitors", PackageName = "bizgaze", ParamName1 = "visitorid")]
        public async Task<ActionResponse<Bizgaze.CRM.Visitors.Infos.Visitor>> VisitorAgreementAsync(ActionRequest arequest, long VisitorId)
        {
            var srequest = SelectRequest.FromRequest(arequest);
            var parameters = new List<Unibase.Data.Parameter>();
            var connection = GetConnection(srequest);
            var visitorinfo = await GetVisitorsAsync(srequest, VisitorId);
            if (visitorinfo.Result != null)
            {
                parameters.Add(new Unibase.Data.Parameter("@visitorid", VisitorId));
                var Agreementupdate = "update bizgazecrm_visitors set agreement=" + "true" + " where visitorid=" + visitorinfo.Result.VisitorId;
                var sqlvoucher = await _dbContext.ExecuteAsync(connection, Agreementupdate, Unibase.Data.Enums.InstructionType.Query, parameters); 
                return ActionResponse<Bizgaze.CRM.Visitors.Infos.Visitor>.CreateSuccessResponse("Agreement Success",null);
            }
            else
            {
                return ActionResponse<Bizgaze.CRM.Visitors.Infos.Visitor>.CreateErrorResponse("", "Visitor Not Found");
            }

        }

        #endregion

        #region check Mobile Number
        /** 
        * Api Name: check Mobile Number 
        * App Name: crm 
        * Description: check Mobile Numbert 
        * Unique Name: bizgaze_transact_visitors_checkMobileNumber 
        * Method Name: checkmobilenumber 
        * Module Name: visitors 
        * Package Name: bizgaze 
        * Param Name1: mobilenumber 
        * Param Name2: emailid   
        * This method is used to check the existing mobile number and email id in the Visitors table and return an error response. 
        */

        [ActionMethod(ApiName = "check Mobile Number", AppName = "crm", Description = "check Mobile Numbert", UniqueName = "bizgaze_transact_visitors_checkMobileNumber", MethodName = "checkmobilenumber", ModuleName = "visitors", PackageName = "bizgaze", ParamName1 = "mobilenumber", ParamName2 = "emailid")]
        public async Task<ActionResponse<Bizgaze.CRM.Visitors.Infos.Visitor>> checkMobileNumberAsync(ActionRequest arequest, string mobileNumber, string emailId)
        {
            await Task.Delay(0);
            var srequest = SelectRequest.FromRequest(arequest);
            var parameters = new List<Unibase.Data.Parameter>();
            var connection = GetConnection(srequest);
            var sql = new Unibase.Data.Sql.Select("bizgazecrm_visitors").AllColumns();
            List<Unibase.Data.Parameter> viparam = new List<Unibase.Data.Parameter>();
            return ActionResponse<Bizgaze.CRM.Visitors.Infos.Visitor>.CreateErrorResponse("", "Mobile Number And Email Id Alredy Exit");
        }

        #endregion

        #region  Getting data from Visitors by Mobile Number
        /** 
         * Get a single visitor record by mobile number 
         * @param SelectRequest srequest - the filer criteria 
         * @param string MobileNumber - the mobile number of the visitor that needs to be retrieved 
         * @return SelectResponse<Bizgaze.CRM.Visitors.Infos.Visitor> - the response containing visitor information 
         */
        [SelectMethod(ApiName = "getvisitorsbymobilenumber", AppName = "crm", Description = "getvisitorsbymobilenumber", UniqueName = "bizgaze_transact_visitors_getvisitorsbymobilenumber", MethodName = "visitorsbymobilenumber", ModuleName = "visitors", PackageName = "bizgaze", ParamName1 = "mobilenumber")]
        public async Task<SelectResponse<Bizgaze.CRM.Visitors.Infos.Visitor>> VisitorsbyMobileNumberAsync(SelectRequest srequest, string MobileNumber)
        {
            var exp = srequest.Filter();
            exp = exp & Exp.Eq(Bizgaze.CRM.Visitors.Infos.Visitor.Property.MobileNumber.ToString(), MobileNumber);
            srequest.Filter(exp);
            return await GetSingleAsync<Bizgaze.CRM.Visitors.Infos.Visitor>(srequest);
        }
        #endregion

        #region  Get installedAppId Async
        /*  
         This method is used to get the install app id based on the visitorid.  
         The method takes the select request and visitorid as parameters and returns a select response of type Bizgaze.CRM.Visitors.Infos.Visitor.  
         */

        [SelectMethod(ApiName = "getinstallappid", AppName = "crm", Description = "getinstallappid", UniqueName = "bizgaze_crm_visitors_getinstallappid", MethodName = "getinstallappid", ModuleName = "visitors", PackageName = "bizgaze", ParamName1 = "visitorid")]
        public async Task<SelectResponse<Bizgaze.CRM.Visitors.Infos.Visitor>> GetInstallappAsync(SelectRequest srequest, long VisitorId)
        {
            var exp = srequest.Filter();
            exp = exp & Exp.Eq(Bizgaze.CRM.Visitors.Infos.Visitor.Property.VisitorId.ToString(), VisitorId);
            srequest.Filter(exp);
            return await GetSingleAsync<Bizgaze.CRM.Visitors.Infos.Visitor>(srequest);
        }
        #endregion

        #region Get Visitor INformation
        // Gets Visitors information with the given Visitor Id.
        public async Task<SelectResponse<Bizgaze.CRM.Visitors.Infos.Visitor>> GetVisitorsAsync(SelectRequest srequest, long VisitorId)
        {
            var exp = srequest.Filter();
            exp &= Exp.Eq(Bizgaze.CRM.Visitors.Infos.Visitor.Property.VisitorId.ToString(), VisitorId);
            srequest.Filter(exp);
            return await GetSingleAsync<Bizgaze.CRM.Visitors.Infos.Visitor>(srequest);
        }

        #endregion

        #region Save Visitor Image
        /* 
         *  ActionMethod Name: SaveVisitorImage 
         *  ApiName: Bizgaze_crm_Visitor_Capture 
         *  Description: This method is used for capturing the image of the visitor. 
         *  AppName: crm 
         *  ModuleName: visitors 
         *  PackageName: bizgaze 
        *  UniqueName: Bizgaze_crm_Visitor_Capture 
       */
        [ActionMethod(ApiName = "VisitorCapture ", AppName = "crm", Description = "image Capture ", UniqueName = "Bizgaze_crm_Visitor_Capture", MethodName = "SaveVisitorImage", ModuleName = "visitors", PackageName = "bizgaze")]
        public async Task<ActionResponse> SaveVisitorImageAsync(ActionRequest request, ExpandoObject obj)
        {
            using (var requestTrans = await request.BeginTransactionAsync())
            {
                {
                    SelectRequest rrequest = new SelectRequest(request.Transaction)
                    {
                        Identity = request.Identity
                    };
                    var dictObj = obj as IDictionary<string, dynamic>;
                    var visitorid = Convert.ToInt64(dictObj["visitorid"].ToString());
                    var Photourl = dictObj["image"].ToString();
                    var parameters = new Dictionary<string, object>();
                    var Update = new Update("bizgazecrm_visitors");
                    parameters.Add("photourl", Photourl);
                    Update.Values(parameters).Where(Exp.Eq("visitorid", visitorid));
                    var ds = await _repository.ExecuteAsync(request, Update);
                    if (requestTrans.StartedLocal) await request.CommitTransactionAsync();
                    return ActionResponse.CreateSuccessResponse("Your Image Saved Successfully", ds.Result);
                }
            }
        }
        #endregion
    }
}

